/***********************************************************************/
/*                                                                     */
/*                           Objective Caml                            */
/*                                                                     */
/*             Damien Doligez, projet Para, INRIA Rocquencourt         */
/*                                                                     */
/*  Copyright 1996 Institut National de Recherche en Informatique et   */
/*  Automatique.  Distributed only by permission.                      */
/*                                                                     */
/***********************************************************************/

/* $Id: major_gc.h,v 1.1 1999/09/30 13:06:17 lefessan Exp $ */

#ifndef _major_gc_
#define _major_gc_

#include "freelist.h"
#include "misc.h"

typedef struct {
  void *block;           /* address of the malloced block this chunk live in */
  asize_t alloc;         /* in bytes, used for compaction */
  asize_t size;          /* in bytes */
  char *next;
} heap_chunk_head;

#define Chunk_size(c) (((heap_chunk_head *) (c)) [-1]).size
#define Chunk_alloc(c) (((heap_chunk_head *) (c)) [-1]).alloc
#define Chunk_next(c) (((heap_chunk_head *) (c)) [-1]).next
#define Chunk_block(c) (((heap_chunk_head *) (c)) [-1]).block

extern int gc_phase;
extern unsigned long allocated_words;
extern unsigned long extra_heap_memory;

/* replace by TRACE_
#define Phase_mark 0
#define Phase_sweep 1
#define Phase_idle 2
*/

#define LEFT_FLAG         1
#define RIGHT_FLAG        2

#define MARK_FLAG        4
#define SWEEP_FLAG        8

#define ROOT_FLAG         16
#define SCION_FLAG        32

#define TRACE_LEFTROOTS   (MARK_FLAG | ROOT_FLAG | LEFT_FLAG)
#define TRACE_LEFTSCIONS  (MARK_FLAG | SCION_FLAG | LEFT_FLAG)
#define TRACE_SWEEPLEFT   (SWEEP_FLAG | LEFT_FLAG)
#define TRACE_RIGHTROOTS  (MARK_FLAG | ROOT_FLAG | RIGHT_FLAG)
#define TRACE_RIGHTSCIONS (MARK_FLAG | ROOT_FLAG | RIGHT_FLAG)
#define TRACE_SWEEPRIGHT  (SWEEP_FLAG | RIGHT_FLAG)
#define TRACE_IDLE        0



#ifdef __alpha
typedef int page_table_entry;
#else
typedef char page_table_entry;
#endif

extern char *heap_start;
extern char *heap_end;
extern unsigned long total_heap_size;
extern page_table_entry *page_table;
extern asize_t page_table_size;
extern asize_t page_low, page_high;
extern char *gc_sweep_hp;
extern char *markhp;
extern char *chunk;
extern char *limit;

#define In_heap 1
#define Not_in_heap 0
#define Page(p) ((unsigned long) (p) >> Page_log)
#define Is_in_heap(p) \
  ((addr)(p) >= (addr)heap_start && (addr)(p) < (addr)heap_end \
   && page_table [Page (p)])


void init_major_heap (asize_t);
asize_t round_heap_chunk_size (asize_t);
void darken (value, value *);
void major_collection_slice (void);
void major_collection (void);
void finish_major_cycle (void);

extern void update_weak_pointers (void);



#endif /* _major_gc_ */
