(*************************************************************)
(*** JBLAST avec j pour Jocaml *******************************)
(*** v 0.97 28/05/98           *******************************)
(*** J.Tordjman                *******************************)
(*** joachim.tordjman@inria.fr *******************************)
(*************************************************************)


open Array
open Graphics
open String

let wait_key() =
  try
    while true do 
      if key_pressed() then if read_key() = 's' then raise End_of_file done
  with
  | End_of_file -> ()
        ;; 

let no_level = ref 0;;    (* no du stage, commence a 0 *)
let nobomb = ref 1;;      (* nb bomb max*)
let tbomb = ref 0;;
(* ici on s'occupe du personnage *)
let xperso = [| 40; 480; 40; 480 |];;     (* coordonnees absolues du perso *)
let yperso = [| 30; 300; 300; 30 |];;
let position = [| 0; 0; 0; 0 |];; (* droite, gauche *)
(*    let old_pos = [| 0; 0; 0; 0 |];;*) (* droite, gauche sauvegarde*)
let vivant = [| 2; 2; 2; 2 |];;
let intensite = [| 1; 1; 1; 1 |];;   (* intensite des bombs *)
let direction = [| 0; 0; 0; 0 |];;   (* ou regarde le perso *)
let nbmort = [| 0; 0; 0; 0|];;       (* affiche le score *) 
(* ici on s'occupe de la bombe *)
     (* coordonnes de la bombe *)
let xbomb = create_matrix 4 10 0;;  
let ybomb = create_matrix 4 10 0;;
    (* pas plus de nobomb bombes a la fois *)
let nbomb = create_matrix 4 10 true;;

let jeton = create_matrix 4 5 0;;

let _ = 
  for i = 0 to 3 do
    jeton.(i).(0) <- xperso.(i);
    jeton.(i).(1) <- yperso.(i);
    jeton.(i).(2) <- position.(i);
    jeton.(i).(3) <- vivant.(i);
    jeton.(i).(4) <- direction.(i);
  done
    ;;
(******************************************************)
(*** PARTIE SERVEUR ***********************************)
(******************************************************)

(******************************************************)
(****** CONVERSION d'un fichier .ppm (RAW) en *********)
(****** color array array *****************************)
(* getint <in_channel> -> int                         *)
(*        sert de parser du fichier ascii             *)
(* convert <string> -> <color array array>            *)
(* convert2 <string * color> -> <color array array>   *)
(* get_tcolor < int * int * int * int * color array array> *)
(*                   -> <color array array>           *)
(******************************************************)      

let rec getint f = 
  try
    let c = input_char f in
    match c with
      'P' | '#' -> input_line f;
        getint f
    | ' ' | '\n'-> getint f
    | _ -> let a = make 3 ' ' in 
      a.[0] <- c;
      let c2 = input_char f in
      match c2 with
        ' ' | '\n' -> a.[2] <- a.[0];
          a.[0] <- '0';
          a.[1] <- '0';
          int_of_string a
      | _ -> a.[1] <- c2;
          let c3 = input_char f in
          match c3 with
            ' ' | '\n' ->
              a.[2] <- a.[1];
              a.[1] <- a.[0];
              a.[0] <- '0';
              int_of_string a
          | _ -> a.[2] <- c3; 
              int_of_string a

  with
  | End_of_file -> close_in f;
      
      Pervasives.exit 0
        ;;


let convert fichier =
  let f=open_in fichier in
  let d  = getint f 
  and  e = getint f 
  and  g = getint f in
  let image = create_matrix e d (rgb 0 0 0) in 
  let c0 = input_byte f in
  for j = 0 to (e-1) do
    for i = 0 to (d-1) do
      let c1 = input_byte f
      and c2 = input_byte f
      and c3 = input_byte f in
      image.(j).(i) <- rgb c1 c2 c3;
    done
  done;
  close_in f;
  image
    ;;


let convert2 fichier trans=
  let f=open_in fichier in
  let d  = getint f 
  and  e = getint f 
  and  g = getint f in
  let image = create_matrix e d (rgb 0 0 0) in 
  let c0 = input_byte f in
  for j = 0 to (e-1) do
    for i = 0 to (d-1) do
      let c1 = input_byte f
      and c2 = input_byte f
      and c3 = input_byte f in
      let col = rgb c1 c2 c3 in
      if col = trans then image.(j).(i) <- transp
      else
        image.(j).(i) <- col;
    done
  done;
  close_in f;
  image
    ;;


let get_tcolor y x dy dx source=
  let arendre = create_matrix dx dy white
  in
  begin
    for ix = 0 to (dx-1) do
      for jy = 0 to (dy-1) do
        arendre.(ix).(jy) <- source.(x+ix).(y+jy)
      done
    done;
    arendre
  end
    ;;


let changecol source col1 col2=
  let x = Array.length source 
  and y = Array.length source.(0)
  in
  for ix = 0 to (x-1) do
    for iy = 0 to (y-1) do
      if source.(ix).(iy) = col1 then source.(ix).(iy) <- col2
    done
  done
    ;;

(*****************************************************)

(*************************************************************)
(* make_level:                                               *)
(*    creer le tableau du stage                              *)
(*    attention, cette fonction change la valeur de no_level *)
(*    <string> -> <int array array>                          *) 
    

let make_level level_file =

  let f=open_in level_file
  and level=create_matrix 14 12 0
  in
  try
    no_level := int_of_string (input_line f);
    for i = 11 downto 0 do
      for j = 0 to 13 do
        let c = input_char f in
        match c with 
          'S'  -> level.(j).(i) <- 1
        | 'D'  ->
            let rnd = Random.int 5 in
	    if rnd = 0 then level.(j).(i) <- 2
	    else  level.(j).(i) <- (4+rnd);
        | 'M'  -> level.(j).(i) <- 3
        | _ -> let c2 = input_char f in
          match c2 with 
            'S'  -> level.(j).(i) <- 1
          | 'D'  -> let rnd = Random.int 5 in
	    if rnd = 0 then level.(j).(i) <- 2
	    else  level.(j).(i) <- (4+rnd);
          | 'M'  -> level.(j).(i) <- 3
          | _    -> (); 
      done
    done;
    close_in f;
    level
  with
  | End_of_file -> 
      close_in f;
      Pervasives.exit 1;;


(******************************************************************)
(******** Ici, on fabrique ce qui va etre exporter sur le web *****)


(*********************** on fabrique le level avec le decors *******)
let level=make_level "level3.dt";;

let level_i = !no_level mod 4;;
let (mur, sol,des ) =
  let image = convert "decors.ppm" in
  let mur = get_tcolor ( (level_i)*40 ) 0 40 30 image in
  let sol = get_tcolor ( (level_i)*40 ) 30 40 30 image in
  let des = get_tcolor ( (level_i)*40 ) 60 40 30 image in
  (mur, sol,des );;

(*********************** on fabrique les perso *********************)


let (tab_perso) =
  let perso = let rose =  rgb 132 0 132 in convert2 "man.ppm" rose
  and tab_perso = 
    let pipo = create_matrix 36 45 white in create_matrix 14 4 pipo
  in 

  tab_perso.(0).(0) <- get_tcolor 0 0 36 45 perso;
  tab_perso.(1).(0) <- get_tcolor 0 45 36 45 perso;
  tab_perso.(2).(0) <- get_tcolor 36 0 36 45 perso;
  tab_perso.(3).(0) <- get_tcolor 36 45 36 45 perso;
  tab_perso.(4).(0) <- get_tcolor 72 0 36 45 perso;
  tab_perso.(5).(0) <- get_tcolor 72 45 36 45 perso;
  tab_perso.(6).(0) <- get_tcolor 108 0 36 45 perso;
  tab_perso.(7).(0) <- get_tcolor 108 45 36 45 perso;
  tab_perso.(8).(0) <- get_tcolor 144 0 36 45 perso;
  tab_perso.(9).(0) <- get_tcolor 144 45 36 45 perso;
  tab_perso.(10).(0) <- get_tcolor 180 0 36 45 perso;
  tab_perso.(11).(0) <- get_tcolor 180 45 36 45 perso;
  tab_perso.(12).(0) <- get_tcolor 216 0 36 45 perso;
  tab_perso.(13).(0) <- get_tcolor 216 45 36 45 perso;

  changecol perso green red;
  changecol perso yellow white;

  tab_perso.(0).(1) <- get_tcolor 0 0 36 45 perso;
  tab_perso.(1).(1) <- get_tcolor 0 45 36 45 perso;
  tab_perso.(2).(1) <- get_tcolor 36 0 36 45 perso;
  tab_perso.(3).(1) <- get_tcolor 36 45 36 45 perso;
  tab_perso.(4).(1) <- get_tcolor 72 0 36 45 perso;
  tab_perso.(5).(1) <- get_tcolor 72 45 36 45 perso;
  tab_perso.(6).(1) <- get_tcolor 108 0 36 45 perso;
  tab_perso.(7).(1) <- get_tcolor 108 45 36 45 perso;
  tab_perso.(8).(1) <- get_tcolor 144 0 36 45 perso;
  tab_perso.(9).(1) <- get_tcolor 144 45 36 45 perso;
  tab_perso.(10).(1) <- get_tcolor 180 0 36 45 perso;
  tab_perso.(11).(1) <- get_tcolor 180 45 36 45 perso;
  tab_perso.(12).(1) <- get_tcolor 216 0 36 45 perso;
  tab_perso.(13).(1) <- get_tcolor 216 45 36 45 perso;

  changecol perso red (rgb 1 1 1);
  changecol perso blue red;

  tab_perso.(0).(2) <- get_tcolor 0 0 36 45 perso;
  tab_perso.(1).(2) <- get_tcolor 0 45 36 45 perso;
  tab_perso.(2).(2) <- get_tcolor 36 0 36 45 perso;
  tab_perso.(3).(2) <- get_tcolor 36 45 36 45 perso;
  tab_perso.(4).(2) <- get_tcolor 72 0 36 45 perso;
  tab_perso.(5).(2) <- get_tcolor 72 45 36 45 perso;
  tab_perso.(6).(2) <- get_tcolor 108 0 36 45 perso;
  tab_perso.(7).(2) <- get_tcolor 108 45 36 45 perso;
  tab_perso.(8).(2) <- get_tcolor 144 0 36 45 perso;
  tab_perso.(9).(2) <- get_tcolor 144 45 36 45 perso;
  tab_perso.(10).(2) <- get_tcolor 180 0 36 45 perso;
  tab_perso.(11).(2) <- get_tcolor 180 45 36 45 perso;
  tab_perso.(12).(2) <- get_tcolor 216 0 36 45 perso;
  tab_perso.(13).(2) <- get_tcolor 216 45 36 45 perso;

  changecol perso white red;
  changecol perso  (rgb 1 1 1) white;
  changecol perso red black;

  tab_perso.(0).(3) <- get_tcolor 0 0 36 45 perso;
  tab_perso.(1).(3) <- get_tcolor 0 45 36 45 perso;
  tab_perso.(2).(3) <- get_tcolor 36 0 36 45 perso;
  tab_perso.(3).(3) <- get_tcolor 36 45 36 45 perso;
  tab_perso.(4).(3) <- get_tcolor 72 0 36 45 perso;
  tab_perso.(5).(3) <- get_tcolor 72 45 36 45 perso;
  tab_perso.(6).(3) <- get_tcolor 108 0 36 45 perso;
  tab_perso.(7).(3) <- get_tcolor 108 45 36 45 perso;
  tab_perso.(8).(3) <- get_tcolor 144 0 36 45 perso;
  tab_perso.(9).(3) <- get_tcolor 144 45 36 45 perso;
  tab_perso.(10).(3) <- get_tcolor 180 0 36 45 perso;
  tab_perso.(11).(3) <- get_tcolor 180 45 36 45 perso;
  tab_perso.(12).(3) <- get_tcolor 216 0 36 45 perso;
  tab_perso.(13).(3) <- get_tcolor 216 45 36 45 perso;
  (tab_perso);;

(*********************** on fabrique les bombs *********************)

let (bombtabp) =
  let bombp = convert2 "bomb.ppm" green
  and bombtabp = 
    let pipo = create_matrix 40 30 white in Array.create 6 pipo
  in 
  bombtabp.(0) <- get_tcolor 0 0 40 30 bombp;
  bombtabp.(1) <- get_tcolor 40 0 40 30 bombp;
  bombtabp.(2) <- get_tcolor 80 0 40 30 bombp;
  bombtabp.(3) <- get_tcolor 120 0 40 30 bombp;
  bombtabp.(4) <- get_tcolor 160 0 40 30 bombp;
  bombtabp.(5) <- get_tcolor 200 0 40 30 bombp;
  (bombtabp);;

(********************* on cree les bonus **************************)

let (itemtabp)=
  let itemp = convert "items.ppm" 
  and itemtabp = Array.create 4 (create_matrix 40 30 white)
  in
  itemtabp.(0) <- get_tcolor 0 0 40 30 itemp;
  itemtabp.(1) <- get_tcolor 40 0 40 30 itemp;

  changecol itemp (rgb 0 181 0) red;
  changecol itemp (rgb 0 255 255) white;
  changecol itemp blue (rgb 132 0 132);
  changecol itemp (rgb 0 115 0) (rgb 200 0 0);
  changecol itemp (rgb 255 255 0) white;
  changecol itemp (rgb 255 132 255) (rgb 255 255 0);
  changecol itemp (rgb 255 0 255) (rgb 200 200 0);
  changecol itemp (rgb 255 222 41) (rgb 225 225 0);

  itemtabp.(2) <- get_tcolor 0 0 40 30 itemp;
  itemtabp.(3) <- get_tcolor 40 0 40 30 itemp;
  (itemtabp);;

(*********************** on fabrique les explosions ***************)

let (exptab) =
  let exptab =
    let pipo = create_matrix 40 30 white in create_matrix 4 10 pipo
  in
  let explo = convert2 "explo1.ppm" green in
  exptab.(0).(0)  <- get_tcolor 0 0 40 30 explo;
  exptab.(1).(0)  <- get_tcolor 40 0 40 30 explo;
  exptab.(2).(0)  <- get_tcolor 80 0 40 30 explo;
  exptab.(3).(0)  <- get_tcolor 120 0 40 30 explo;

  let explo = convert2 "explo2.ppm" green in
  exptab.(0).(1)  <- get_tcolor 0 0 40 30 explo;
  exptab.(1).(1)  <- get_tcolor 40 0 40 30 explo;
  exptab.(2).(1)  <- get_tcolor 80 0 40 30 explo;
  exptab.(3).(1)  <- get_tcolor 120 0 40 30 explo;

  let explo = convert2 "explo3.ppm" green in
  exptab.(0).(2)  <- get_tcolor 0 0 40 30 explo;
  exptab.(1).(2)  <- get_tcolor 40 0 40 30 explo;
  exptab.(2).(2)  <- get_tcolor 80 0 40 30 explo;
  exptab.(3).(2)  <- get_tcolor 120 0 40 30 explo;

  let explo = convert2 "explo4.ppm" green in
  exptab.(0).(3)  <- get_tcolor 0 0 40 30 explo;
  exptab.(1).(3)  <- get_tcolor 40 0 40 30 explo;
  exptab.(2).(3)  <- get_tcolor 80 0 40 30 explo;
  exptab.(3).(3)  <- get_tcolor 120 0 40 30 explo;

  let explo = convert2 "explo5.ppm" green in
  exptab.(0).(4)  <- get_tcolor 0 0 40 30 explo;
  exptab.(1).(4)  <- get_tcolor 40 0 40 30 explo;
  exptab.(2).(4)  <- get_tcolor 80 0 40 30 explo;
  exptab.(3).(4)  <- get_tcolor 120 0 40 30 explo;

  let explo = convert2 "explo6.ppm" green in
  exptab.(0).(5)  <- get_tcolor 0 0 40 30 explo;
  exptab.(0).(6)  <- get_tcolor 40 0 40 30 explo;
  exptab.(0).(7)  <- get_tcolor 80 0 40 30 explo;
  exptab.(0).(8)  <- get_tcolor 120 0 40 30 explo;

  let explo = convert2 "explo7.ppm" green in
  exptab.(1).(5)  <- get_tcolor 0 0 40 30 explo;
  exptab.(1).(6)  <- get_tcolor 40 0 40 30 explo;
  exptab.(1).(7)  <- get_tcolor 80 0 40 30 explo;
  exptab.(1).(8)  <- get_tcolor 120 0 40 30 explo;

  let explo = convert2 "explo8.ppm" green in
  exptab.(2).(5)  <- get_tcolor 0 0 40 30 explo;
  exptab.(2).(6)  <- get_tcolor 40 0 40 30 explo;
  exptab.(2).(7)  <- get_tcolor 80 0 40 30 explo;
  exptab.(2).(8)  <- get_tcolor 120 0 40 30 explo;

  let explo = convert2 "explo9.ppm" green in
  exptab.(3).(5)  <- get_tcolor 0 0 40 30 explo;
  exptab.(3).(6)  <- get_tcolor 40 0 40 30 explo;
  exptab.(3).(7)  <- get_tcolor 80 0 40 30 explo;
  exptab.(3).(8)  <- get_tcolor 120 0 40 30 explo;

  let explo = convert2 "explo10.ppm" green in
  exptab.(0).(9)  <- get_tcolor 0 0 40 30 explo;
  exptab.(1).(9)  <- get_tcolor 40 0 40 30 explo;
  exptab.(2).(9)  <- get_tcolor 80 0 40 30 explo;
  exptab.(3).(9)  <- get_tcolor 120 0 40 30 explo;

  (exptab);;

(****************** FIN de ce qui va etre exporte - donnees ****)
(***************************************************************)



(******************* les fonctions qui suivent vont migrer ******)


let affiche_level level=
  clear_graph();
  set_color black;
  fill_rect 0 0 560 380;
  set_color white;
  moveto 20 365;
  draw_string "Joueur 1:";
  moveto 120 365;
  draw_string "Joueur 2:";
  moveto 220 365;
  draw_string "Joueur 3:";
  moveto 320 365;
  draw_string "Joueur 4:";
  for jo = 0 to 3 do
    moveto (80+jo*100) 365;
    draw_string (string_of_int nbmort.(jo));
  done;
  let imur = make_image mur
  and isol = make_image sol
  and ides = make_image des
(*  and item = make_image itemtabp.(0)
    and item2 = make_image itemtabp.(1)
    and item3 = make_image itemtabp.(2)
    and item4 = make_image itemtabp.(3)*)
  in
  for i = 0 to 11 do
    for j = 0 to 13 do
      let c = level.(j).(i) in
      match c with 
        1  -> draw_image isol (40*j) (30*i)
      | 2 | 5 | 6 | 7 | 8  -> draw_image ides (40*j) (30*i)
      | 3  -> draw_image imur (40*j) (30*i)
(*  | 9  -> draw_image item (40*j) (30*i)
    | 10  -> draw_image item2 (40*j) (30*i)
    | 11  -> draw_image item3 (40*j) (30*i)
    | 12  -> draw_image item4 (40*j) (30*i) *)
      | _  -> 
          Pervasives. exit 1 
    done
  done;
  ;;


(***************************************************************)
(***** Ce qui est propre au serveur *********)

let def sync2! (tab,i) =
  {jeton.(i) <- tab;} 


let def join_game () | free! (n) =
free (n+1)|
if n > 3 then reply -1 to join_game
else {{vivant.(n) <- 1; (jeton.(n)).(3)<- 1;} | reply n to join_game}
  or join_game () | nomore! (n) =
  nomore (n) | reply -1 to join_game
  or start () | nomore! (n) = 
  nomore(n) | reply n to start

  or start () | free! (n) =
  nomore(n) | reply n to start

(* signaux bombs *)
let def isbomb3 () | bombis3! (tb,i,bombio,xc,yc : int*int*int*int*int) =
 reply (tb,i,bombio,xc,yc) to isbomb3
let def isbomb2 () | bombis2! (tb,i,bombio,xc,yc : int*int*int*int*int) =
reply (tb,i,bombio,xc,yc) to isbomb2
let def isbomb1 () | bombis1! (tb,i,bombio,xc,yc : int*int*int*int*int) =
reply (tb,i,bombio,xc,yc) to isbomb1
let def isbomb0 () | bombis0! (tb,i,bombio,xc,yc : int*int*int*int*int) =
reply (tb,i,bombio,xc,yc) to isbomb0

let def bombis! (tb,i,bombio,xc,yc : int*int*int*int*int) =
  {if jeton.(0).(3) < 2 then bombis0 (tb,i,bombio,xc,yc)} |
  {if jeton.(1).(3) < 2 then bombis1 (tb,i,bombio,xc,yc)} |
  {if jeton.(2).(3) < 2 then bombis2 (tb,i,bombio,xc,yc)} |
  {if jeton.(3).(3) < 2 then bombis3 (tb,i,bombio,xc,yc)}

(* signaux level *)
let def islevel3 () | levelis3! (xl,yl,case : int*int*int) = 
reply (xl,yl,case) to islevel3
let def islevel2 () | levelis2! (xl,yl,case : int*int*int) = 
reply (xl,yl,case) to islevel2
let def islevel1 () | levelis1! (xl,yl,case : int*int*int) = 
reply (xl,yl,case) to islevel1
let def islevel0 () | levelis0! (xl,yl,case : int*int*int) = 
reply (xl,yl,case) to islevel0

let def levelisc! (xl,yl,case : int*int*int) =
  {if jeton.(0).(3) < 2 then levelis0 (xl,yl,case)} |
  {if jeton.(1).(3) < 2 then levelis1 (xl,yl,case)} |
  {if jeton.(2).(3) < 2 then levelis2 (xl,yl,case)} |
  {if jeton.(3).(3) < 2 then levelis3 (xl,yl,case)}

(* sigaux item *)
let def isitem3 () | itemis3! (nj,intens : int*int) = 
reply (nj,intens) to isitem3
let def isitem2 () | itemis2! (nj,intens : int*int) = 
reply (nj,intens) to isitem2
let def isitem1 () | itemis1! (nj,intens : int*int) = 
reply (nj,intens) to isitem1
let def isitem0 () | itemis0! (nj,intens : int*int) = 
reply (nj,intens) to isitem0

let def itemis! (nj,intens :int*int) =
  {if jeton.(0).(3) < 2 then itemis0 (nj,intens)} |
  {if jeton.(1).(3) < 2 then itemis1 (nj,intens)} |
  {if jeton.(2).(3) < 2 then itemis2 (nj,intens)} |
  {if jeton.(3).(3) < 2 then itemis3 (nj,intens)}

(*let def dead (i:int) =
  {(jeton.(i)).(3)<- 0;
  vivant.(i) <- 0;
  }*)
(*let def live (i:int) =
  {*)
(*(jeton.(i)).(2) <-13;*)
(*(jeton.(i)).(3)<- 1;
  vivant.(i) <- 1;
  }*)

let def imoff! (i:int) =
  {
  (
  jeton.(i)).(3)<-2;
  print_string ("le joueur "^(string_of_int (i))^" nous a quitte");
  print_newline();
}

let def getdead () =
  reply vivant to getdead

(*let def sync (tab,i) =
  jeton.(i) <- tab;
  reply jeton to sync
  *)

let def persochk (i) =
  reply jeton.(i)

    ;;

spawn {free 0};;





(***********************************************)
(*** FIN DE LA PARTIE SERVER *******************)
(***********************************************)



let start_client () =
  begin

    let nbjoueur = join_game () in
    if nbjoueur < 0 then 
      begin
        print_string "je suis desole mais toutes les places sont prises";
        print_int nbjoueur;
        print_newline ();
        Pervasives.exit 0;
      end;
    vivant.(nbjoueur) <- 1;  
    print_string "je suis le joueur no ";
    print_int (nbjoueur+1);
    print_newline ();
    



    print_string "salut je suis le client";
    print_newline();
    open_graph (" 560x380+30+30");
    set_color black;
    fill_rect 0 0 560 380;
    set_color white;
    moveto 100 320;
    draw_string "Bienvenue a ";
    moveto 100 300;
    set_color yellow;
    draw_string "      ____.         __________ .__                     __   ";
    moveto 100 290;
    set_color (rgb 255 200 0);
    draw_string "     |    |         \______   \|  |  _____     _______/  |_ ";
    moveto 100 280;
    set_color (rgb 255 150 0);
    draw_string "     |    |  ______  |    |  _/|  |  \__  \   /  ___/\   __\ ";
    moveto 100 270;
    set_color (rgb 255 100 0);
    draw_string " /\__|    | /_____/  |    |   \|  |__ / __ \_ \___ \  |  |  ";
    moveto 100 260;
    set_color (rgb 255 50 0);
    draw_string " \________|          |______  /|____/(____  //____  > |__|  ";
    moveto 100 250;
    set_color (rgb 255 0 0);
    draw_string "                            \/            \/      \/        ";
    set_color (rgb 0 255 255);
    moveto 50 150;
    draw_string "Appuyer sur 's' pour commencer...";
    moveto 50 130;
    draw_string "Les touches pour jouer :   8,6,2,4 pour les directions";
    moveto 50 115;
    draw_string "                           ' ' pour deposer les bombes";
    moveto 50 100;
    draw_string "                           0 pour les detonateurs";
    moveto 50 85;
    draw_string "                           . pour recommancer";

    wait_key();
    let nbfinal = (start()-1) in 
    print_string "nombre de joueur :";
    print_int nbfinal;
    print_newline();
(* les constante graphiques *)
    let tabperso = create_matrix 14 4 (create_image 36 45)
    in 
    for ix = 0 to 13 do
      for iy = 0 to 3 do
        tabperso.(ix).(iy) <- make_image tab_perso.(ix).(iy)
      done
    done;

    let explotab = create_matrix 4 10 (create_image 40 30) 
    in
    for ix = 0 to 9 do
      for iy = 0 to 3 do
        explotab.(iy).(ix) <- make_image exptab.(iy).(ix)
      done
    done;

    let itemstab = Array.create 4 (create_image 40 30) 
    in
    for ix = 0 to 3 do
      itemstab.(ix) <- make_image itemtabp.(ix)
    done;

    let bombtab = Array.create 6 (create_image 40 30) 
    in
    for ix = 0 to 5 do
      bombtab.(ix) <- make_image bombtabp.(ix)
    done;

    let isol = make_image sol in
(* fin des ctes gfx *)

(* les variables: cf tout en haut *)
    let safe = Array.create 4 (create_image 36 45) in 

(* les fonctions *)


    let refresh (i) =
      let provis = Array.create 5 0 in
      provis.(0) <- xperso.(i);
      provis.(1) <- yperso.(i);
      provis.(2) <- position.(i);
      provis.(3) <- vivant.(i);
      provis.(4) <- direction.(i);

      spawn {sync2 (provis,i)};

    in 

(* ici on commence l'affichage *)
    print_string "je narrive pas ici";
    print_newline ();
    affiche_level level;

    for js = 0 to 3 do
      safe.(js) <- create_image 36 45;
    done;

    let actitem xo yo i=
      let c = level.(xo).(yo) in
      begin
        level.(xo).(yo)<-1;
        match c with
          9  -> tbomb := 0;
            nobomb := !nobomb +1;
        | 10 -> if intensite.(i) < 8 then intensite.(i) <- intensite.(i) +1;
            spawn {itemis (i,intensite.(i))};
        | 11 -> tbomb := 1;
            nobomb := 1;
        | _  -> if intensite.(i) < 8 then intensite.(i) <- intensite.(i)*2;
            spawn {itemis (i,intensite.(i))};

      end;
      spawn {levelisc (xo,yo,1)};

    in

    let check_position dx dy i=
      let xprime2 dx2 = (xperso.(i) +  dx2 +dx) / 40
      and yprime2 dy2 = (yperso.(i) +  dy2 +dy) / 30
      in
      if level.(xprime2 0).(yprime2 0) > 8 then actitem (xprime2 0) (yprime2 0) i;
      if level.(xprime2 20).(yprime2 0) > 8 then actitem (xprime2 20) (yprime2 0) i;
      if level.(xprime2 20).(yprime2 20) > 8 then actitem (xprime2 20) (yprime2 20) i;
      if level.(xprime2 0).(yprime2 20) > 8 then actitem (xprime2 0) (yprime2 20) i;


      (   (level.(xprime2 0).(yprime2 0) = 1)
         && (level.(xprime2 20).(yprime2 0) = 1)
         && (level.(xprime2 20).(yprime2 20) = 1)
         && (level.(xprime2 0).(yprime2 20) = 1))
    ||
      ( ( dx>0 )
         && (level.(xprime2 0).(yprime2 0) = 4)
         && (level.(xprime2 0).(yprime2 20) = 4)
         && (level.(xprime2 20).(yprime2 0) = 1)
         && (level.(xprime2 20).(yprime2 20) = 1))
    ||
      ( ( dx<0 )
         && (level.(xprime2 0).(yprime2 0) = 1)
         && (level.(xprime2 0).(yprime2 20) = 1)
         && (level.(xprime2 20).(yprime2 0) = 4)
         && (level.(xprime2 20).(yprime2 20) = 4))
    ||
      ( ( dy>0 )
         && (level.(xprime2 0).(yprime2 0) = 4)
         && (level.(xprime2 20).(yprime2 0) = 4)
         && (level.(xprime2 0).(yprime2 20) = 1)
         && (level.(xprime2 20).(yprime2 20) = 1))
    ||
      ( ( dy<0 )
         && (level.(xprime2 0).(yprime2 0) = 1)
         && (level.(xprime2 20).(yprime2 0) = 1)
         && (level.(xprime2 0).(yprime2 20) = 4)
         && (level.(xprime2 20).(yprime2 20) = 4))

    in

    let check other i =
      if other = i || vivant.(other) = 2 then false
      else
        (   (xperso.(i) > (xperso.(other) - 36))
           && (xperso.(i) < (xperso.(other) + 36))
           && (yperso.(i) > (yperso.(other) - 45))
           && (yperso.(i) < (yperso.(other) + 45))
           )


    in
    let checkitfirst i =
      let cnt = ref 0 in
      for other = 0 to 3 do 
        if (check other i) then
          begin
(*  print_string "dedans";
    print_newline();*)
            cnt := !cnt +1;
            draw_image safe.(other) xperso.(other) yperso.(other);
            blit_image safe.(i) xperso.(i) yperso.(i);
            blit_image safe.(other) xperso.(other) yperso.(other);
            draw_image tabperso.(position.(other)+direction.(other)).(other) xperso.(other) yperso.(other);
          end;
      done;
      if !cnt = 0 then   blit_image safe.(i) xperso.(i) yperso.(i);
    in 

    let refresh_scr i =
      checkitfirst i;
      if vivant.(i) = 1 then draw_image tabperso.(position.(i)+direction.(i)).(i) xperso.(i) yperso.(i) 
      else draw_image tabperso.(8).(i) xperso.(i) yperso.(i);
      refresh(i);
    in

    let droite i =
      draw_image safe.(i) xperso.(i) yperso.(i);
      if check_position 20 0 i then xperso.(i) <- xperso.(i) + 20;
(*  blit_image safe.(i) xperso.(i) yperso.(i);*)
      if position.(i) = 0 then position.(i)<-1
      else position.(i)<-0;
      direction.(i)<-6;
      refresh_scr i;
    in
    let gauche i =
      draw_image safe.(i) xperso.(i) yperso.(i);
      if check_position (-20) 0 i then xperso.(i) <- xperso.(i) - 20;
(* checkitfirst i;
   blit_image safe.(i) xperso.(i) yperso.(i);*)
      if position.(i) = 0 then position.(i)<-1
      else position.(i)<-0;
      direction.(i)<-2;
  refresh_scr i;
in
let descent i =
  draw_image safe.(i) xperso.(i) yperso.(i);
  if check_position 0 (-15) i then yperso.(i) <- yperso.(i) - 15;
(* checkitfirst i;
  blit_image safe.(i) xperso.(i) yperso.(i);*)
  if position.(i) = 0 then position.(i)<-1
                    else position.(i)<-0;
  direction.(i)<-4;
  refresh_scr i;
in
let monte i =
  draw_image safe.(i) xperso.(i) yperso.(i);
  if check_position 0 15 i then yperso.(i) <- yperso.(i) + 15;
(* checkitfirst i;
  blit_image safe.(i) xperso.(i) yperso.(i);*)
  if position.(i) = 0 then position.(i)<-1
                    else position.(i)<-0;
  direction.(i)<-0;
  refresh_scr i;
in


let raster i =
  while true do
    Thread.delay 0.0;
    for j = 0 to nbfinal do
      if j = i then () else
      begin
        let persochkt = persochk(j) in
        if (xperso.(j) = persochkt.(0)) && (yperso.(j) = persochkt.(1)) 
            && (persochkt.(3) = vivant.(j)) && (persochkt.(4) = direction.(j))
            && (persochkt.(2) = position.(j)) then () else 
          begin
            if (persochkt.(3) != vivant.(j)) && (vivant.(j)=1) then 
              begin
                set_color black;
                moveto (80+j*100) 365;
                draw_string (string_of_int nbmort.(j));
                set_color white;
                moveto (80+j*100) 365;
                nbmort.(j) <- nbmort.(j)+1;
                draw_string (string_of_int nbmort.(j));
              end;
            vivant.(j)<- persochkt.(3);
            position.(j)<- persochkt.(2);
            direction.(j)<- persochkt.(4);
            draw_image safe.(j) xperso.(j) yperso.(j);
            xperso.(j) <- persochkt.(0);
            yperso.(j) <- persochkt.(1);
            if vivant.(j)<2 then
              begin
(*blit_image safe.(j) xperso.(j) yperso.(j);*)
                checkitfirst (j);
                if vivant.(j)=1 then draw_image tabperso.(position.(j)+direction.(j)).(j) xperso.(j) yperso.(j)
                else  draw_image tabperso.(8).(j) xperso.(j) yperso.(j);
              end
          end
      end
    done
  done
in

(* ici on gere les bombes *)



let is_dead xid yid i =
  (xperso.(i) < (xid*40 +40)) && (xperso.(i) > (xid*40 - 39))
    && (yperso.(i) < (yid*30 +30)) && (yperso.(i) > (yid*30 - 29))
in




let explosion i bombi=
  let xb = xbomb.(i).(bombi)/40
  and yb = ybomb.(i).(bombi)/30
  and touche joueurs =
    draw_image safe.(joueurs) xperso.(joueurs) yperso.(joueurs);
    vivant.(joueurs) <- 0;
    direction.(joueurs) <- 8;
    set_color black;
    moveto (80+joueurs*100) 365;
    draw_string (string_of_int nbmort.(joueurs));
    set_color white;
    moveto (80+joueurs*100) 365;
    nbmort.(joueurs) <- nbmort.(joueurs)+1;
    draw_string (string_of_int nbmort.(joueurs));

  in
(* on regarde si les joueurs sont tjs vivant *)
(*centre*)
(* for pl = 0 to 3 do    on decide de tester seulement si ce joueur est mort*)
  if (vivant.(nbjoueur)=1) && (is_dead xb yb nbjoueur )
  then touche(nbjoueur);
(*   done;*)
(*droite*)
  begin
    let rec right p =
      if p>0 && ((xb+p)<14) then
        let case = level.(xb+p).(yb) in
        if case = 1 then
          begin
            if (vivant.(nbjoueur)=1) &&
              (is_dead (xb+p) yb nbjoueur )
            then 
              touche(nbjoueur);
            right (p-1)
          end
        else
          if case = 4 then 
            level.(xb+p).(yb) <- 1
    in
    right  intensite.(i)
  end;
(*gauche*)
  begin
    try
      for p = 1 to intensite.(i) do
        if ((xb-p)>=0) then
          match level.(xb-p).(yb)  with
            1 ->(* for pl = 0 to 3 do*)
              if (vivant.(nbjoueur)=1) &&
                (is_dead (xb-p) yb  nbjoueur)
              then touche(nbjoueur);
      (* done;*)
          |  4 -> level.(xb-p).(yb) <- 1;
              raise (End_of_file);
          | _ ->  raise (End_of_file);

      done;
    with
    | End_of_file -> ();
  end;
(*haut*)
  begin
    try
      for p = 1 to intensite.(i) do
        if ((yb+p)<12) then
          match level.(xb).(yb+p)  with
            1 -> (*for pl = 0 to 3 do*)
              if (vivant.(nbjoueur)=1) &&
                (is_dead xb (yb+p) nbjoueur )
              then touche(nbjoueur);
       (*done;*)
          |  4 -> level.(xb).(yb+p) <- 1;
              raise (End_of_file);
          | _ ->  raise (End_of_file);

      done;
    with
    | End_of_file -> ();
  end;
(*bas*)
  begin
    try
      for p = 1 to intensite.(i) do
        if ((yb-p)>=0) then
          match level.(xb).(yb-p)  with
            1 -> (*for pl = 0 to 3 do*)
              if (vivant.(nbjoueur)=1) &&
                (is_dead xb (yb-p) nbjoueur )
              then touche(nbjoueur);
       (*done;*)
          |  4 -> level.(xb).(yb-p) <- 1;
              raise (End_of_file);
          | _ ->  raise (End_of_file);

      done;
    with
    | End_of_file -> ();
  end;

  for j = 0 to 3 do
(*case centrale*)
    draw_image explotab.(j).(0) xbomb.(i).(bombi) ybomb.(i).(bombi);
(* droite *)
    begin
      try
        for p = 1 to (intensite.(i)-1) do
          if ((xb+p)<14) then
            match level.(xb+p).(yb) with
              1 -> draw_image explotab.(j).(6) (xbomb.(i).(bombi)+40*p) ybomb.(i).(bombi)
            | 3 -> raise (End_of_file);
            | _ -> draw_image explotab.(j).(9) (xbomb.(i).(bombi)+40*p) ybomb.(i).(bombi);
                raise (End_of_file);
        done;

        let p = intensite.(i) in
        if ((xb+p)<14) then 
          match level.(xb+p).(yb) with
            1 -> draw_image explotab.(j).(8) (xbomb.(i).(bombi)+40*p) ybomb.(i).(bombi)
          | 3 -> raise (End_of_file);
          | _ -> draw_image explotab.(j).(9) (xbomb.(i).(bombi)+40*p) ybomb.(i).(bombi);
              raise (End_of_file);
      with
      | End_of_file -> ();
    end;
(* gauche *)
    begin
      try
        for p = 1 to (intensite.(i)-1) do
          if ((xb-p)>=0) then 
            match level.(xb-p).(yb) with
              1 -> draw_image explotab.(j).(7) (xbomb.(i).(bombi)-40*p) ybomb.(i).(bombi)
            | 3 -> raise (End_of_file);
            | _ -> draw_image explotab.(j).(9) (xbomb.(i).(bombi)-40*p) ybomb.(i).(bombi);
                raise (End_of_file);
        done;

        let p = intensite.(i) in
        if ((xb-p)>=0) then
          match level.(xb-p).(yb) with
            1 -> draw_image explotab.(j).(5) (xbomb.(i).(bombi)-40*p) ybomb.(i).(bombi)
          | 3 -> raise (End_of_file);
          | _ -> draw_image explotab.(j).(9) (xbomb.(i).(bombi)-40*p) ybomb.(i).(bombi);
              raise (End_of_file);
      with
      | End_of_file -> ();
    end;
(* haut *)
    begin
      try
        for p = 1 to (intensite.(i)-1) do
          if ((yb+p)<12) then 
            match level.(xb).(yb+p) with
              1 -> draw_image explotab.(j).(4) xbomb.(i).(bombi) (ybomb.(i).(bombi)+30*p)
            | 3 -> raise (End_of_file);
            | _ -> draw_image explotab.(j).(9) xbomb.(i).(bombi) (ybomb.(i).(bombi)+30*p);
                raise (End_of_file);
        done;
        let p = intensite.(i) in
        if ((yb+p)<12) then
          match level.(xb).(yb+p) with
            1 -> draw_image explotab.(j).(3) xbomb.(i).(bombi) (ybomb.(i).(bombi)+30*p)
          | 3 -> raise (End_of_file);
          | _ -> draw_image explotab.(j).(9) xbomb.(i).(bombi) (ybomb.(i).(bombi)+30*p);
              raise (End_of_file);
      with
      | End_of_file -> ();
    end;
(* bas *)
    begin
      try
        for p = 1 to (intensite.(i)-1) do
          if ((yb-p)>=0) then
            match level.(xb).(yb-p) with
              1 -> draw_image explotab.(j).(1) xbomb.(i).(bombi) (ybomb.(i).(bombi)-30*p)
            | 3 -> raise (End_of_file);
            | _ -> draw_image explotab.(j).(9) xbomb.(i).(bombi) (ybomb.(i).(bombi)-30*p);
                raise (End_of_file);
        done;
        let p = intensite.(i) in
        if ((yb-p)>=0) then 
          match level.(xb).(yb-p) with
            1 -> draw_image explotab.(j).(2) xbomb.(i).(bombi) (ybomb.(i).(bombi)-30*p)
          | 3 -> raise (End_of_file);
          | _ -> draw_image explotab.(j).(9) xbomb.(i).(bombi) (ybomb.(i).(bombi)-30*p);
              raise (End_of_file);

      with
      | End_of_file -> ();
    end;

    Thread.delay 0.1;

(* a partir d'ici on restaure *)
(*case centrale*)
    draw_image isol xbomb.(i).(bombi) ybomb.(i).(bombi);
(* droite *)
    begin
      try
        for p = 1 to intensite.(i) do
          if ((xb+p)<14) then
            match level.(xb+p).(yb) with 
              1 -> draw_image isol (xbomb.(i).(bombi)+40*p) ybomb.(i).(bombi)
            | 3 -> raise (End_of_file);
            | _ -> draw_image isol (xbomb.(i).(bombi)+40*p) ybomb.(i).(bombi);
                raise (End_of_file);

        done;
      with
      | End_of_file -> ();
    end;
(* gauche *)
    begin
      try
        for p = 1 to intensite.(i) do
          if ((xb-p)>=0) then
            match level.(xb-p).(yb) with 
              1 -> draw_image isol (xbomb.(i).(bombi)-40*p) ybomb.(i).(bombi)
            | 3 -> raise (End_of_file);
            | _ -> draw_image isol (xbomb.(i).(bombi)-40*p) ybomb.(i).(bombi);
                raise (End_of_file);

        done;
      with
      | End_of_file -> ();
    end;
(* haut *)
    begin
      try
        for p = 1 to intensite.(i) do
          if ((yb+p)<12) then 
            match level.(xb).(yb+p) with 
              1 -> draw_image isol xbomb.(i).(bombi) (ybomb.(i).(bombi)+30*p)
            | 3 -> raise (End_of_file);
            | _ -> draw_image isol xbomb.(i).(bombi) (ybomb.(i).(bombi)+30*p);
                raise (End_of_file);


        done;
      with
      | End_of_file -> ();
    end;
(* bas *)
    begin
      try
        for p = 1 to intensite.(i) do
          if ((yb-p)>=0) then
            match level.(xb).(yb-p) with 
              1 -> draw_image isol xbomb.(i).(bombi) (ybomb.(i).(bombi)-30*p)
            | 3 -> raise (End_of_file);
            | _ -> draw_image isol xbomb.(i).(bombi) (ybomb.(i).(bombi)-30*p);
                raise (End_of_file);

        done;
      with
      | End_of_file -> ();
    end;

(* Thread.delay 0.1;*)
    
  done;
(*centre*)
  level.(xb).(yb) <- 1;
(*droite*)
  begin
    try
      for p = 1 to intensite.(i) do
        if ((xb+p)<14) then
          match level.(xb+p).(yb)  with
            1 -> ();
          | 2 | 9 | 10 | 11 | 12  ->  level.(xb+p).(yb) <-1;
              raise (End_of_file);
          | 5 ->  level.(xb+p).(yb) <-9;
              draw_image itemstab.(0) (xbomb.(i).(bombi)+40*p) ybomb.(i).(bombi);
              raise (End_of_file);
          | 6 ->  level.(xb+p).(yb) <-10;
              draw_image itemstab.(1) (xbomb.(i).(bombi)+40*p) ybomb.(i).(bombi);
              raise (End_of_file);
          | 7 ->  level.(xb+p).(yb) <-11;
              draw_image itemstab.(2) (xbomb.(i).(bombi)+40*p) ybomb.(i).(bombi);
              raise (End_of_file);
          | 8 ->  level.(xb+p).(yb) <-12;
              draw_image itemstab.(3) (xbomb.(i).(bombi)+40*p) ybomb.(i).(bombi);
              raise (End_of_file);
          | _ ->  raise (End_of_file);

      done;
    with
    | End_of_file -> ();
  end;
(*gauche*)
  begin
    try
      for p = 1 to intensite.(i) do
        if ((xb-p)>=0) then
          match level.(xb-p).(yb)  with
            1 -> ();
          | 2 | 9 | 10 | 11 | 12 ->  level.(xb-p).(yb) <-1;
              raise (End_of_file);
          | 5 ->  level.(xb-p).(yb) <-9;
              draw_image itemstab.(0) (xbomb.(i).(bombi)-40*p) ybomb.(i).(bombi);
              raise (End_of_file);
          | 6 ->  level.(xb-p).(yb) <-10;
              draw_image itemstab.(1) (xbomb.(i).(bombi)-40*p) ybomb.(i).(bombi);
              raise (End_of_file);
          | 7 ->  level.(xb-p).(yb) <-11;
              draw_image itemstab.(2) (xbomb.(i).(bombi)-40*p) ybomb.(i).(bombi);
              raise (End_of_file);
          | 8 ->  level.(xb-p).(yb) <-12;
              draw_image itemstab.(3) (xbomb.(i).(bombi)-40*p) ybomb.(i).(bombi);
              raise (End_of_file);
          | _ ->  raise (End_of_file);

      done;
    with
    | End_of_file -> ();
  end;
(*haut*)
  begin
    try
      for p = 1 to intensite.(i) do
        if ((yb+p)<12) then
          match level.(xb).(yb+p)  with
            1 -> ();
          | 2 | 9 | 10 | 11 | 12 ->  level.(xb).(yb+p) <-1;
              raise (End_of_file);
          | 5 -> level.(xb).(yb+p)<-9;
              draw_image itemstab.(0) xbomb.(i).(bombi) (ybomb.(i).(bombi)+30*p);
              raise (End_of_file);
          | 6 -> level.(xb).(yb+p)<-10;
              draw_image itemstab.(1) xbomb.(i).(bombi) (ybomb.(i).(bombi)+30*p);
              raise (End_of_file);
          | 7 -> level.(xb).(yb+p)<-11;
              draw_image itemstab.(2) xbomb.(i).(bombi) (ybomb.(i).(bombi)+30*p);
              raise (End_of_file);
          | 8 -> level.(xb).(yb+p)<-12;
              draw_image itemstab.(3) xbomb.(i).(bombi) (ybomb.(i).(bombi)+30*p);
              raise (End_of_file);
          | _ ->  raise (End_of_file);

      done;
    with
    | End_of_file -> ();
  end;
(*bas*)
  begin
    try
      for p = 1 to intensite.(i) do
        if ((yb-p)>=0) then
          match level.(xb).(yb-p)  with
            1 -> ();
          | 2 | 9 | 10 | 11 | 12->  level.(xb).(yb-p) <-1;
              raise (End_of_file);
          | 5 -> level.(xb).(yb-p)<-9;
              draw_image itemstab.(0) xbomb.(i).(bombi) (ybomb.(i).(bombi)-30*p);
              raise (End_of_file);
          | 6 -> level.(xb).(yb-p)<-10;
              draw_image itemstab.(1) xbomb.(i).(bombi) (ybomb.(i).(bombi)-30*p);
              raise (End_of_file);
          | 7 -> level.(xb).(yb-p)<-11;
              draw_image itemstab.(2) xbomb.(i).(bombi) (ybomb.(i).(bombi)-30*p);
              raise (End_of_file);
          | 8 -> level.(xb).(yb-p)<-12;
              draw_image itemstab.(3) xbomb.(i).(bombi) (ybomb.(i).(bombi)-30*p);
              raise (End_of_file);
          | _ ->  raise (End_of_file);
              
      done;
    with
    | End_of_file -> ();
  end;

  nbomb.(i).(bombi) <- true;
(*for joueur = 0 to 3 do*)
  if vivant.(nbjoueur) != 2 then 
    begin
      draw_image safe.(nbjoueur) xperso.(nbjoueur) yperso.(nbjoueur);
      Thread.delay 0.001;
      refresh_scr (nbjoueur);
    end;
  Thread.exit ()
in



let meanwhile (i,bombi)= 
 let bomb_index = ref 1 in
while true do
   Thread.delay 0.5;
   draw_image isol xbomb.(i).(bombi) ybomb.(i).(bombi);
   bomb_index:= !bomb_index +1;
   if level.(xbomb.(i).(bombi)/40).(ybomb.(i).(bombi)/30) != 4 then 
     explosion i bombi;
   match (!bomb_index) with
   1 | 5 | 9   ->  draw_image bombtab.(1) xbomb.(i).(bombi) ybomb.(i).(bombi)
|  2 | 6 | 10  ->  draw_image bombtab.(2) xbomb.(i).(bombi) ybomb.(i).(bombi)
|  3 | 7 | 11  ->  draw_image bombtab.(3) xbomb.(i).(bombi) ybomb.(i).(bombi)
|  4 | 8 | 12  ->  draw_image bombtab.(0) xbomb.(i).(bombi) ybomb.(i).(bombi)
|  _ ->  explosion i bombi
done

in

let meanwhile2 (i,bombi)= 
 let bomb_index = ref true in
while true do
   Thread.delay 0.5;
   draw_image isol xbomb.(i).(bombi) ybomb.(i).(bombi);
   bomb_index:= not !bomb_index ;
   if level.(xbomb.(i).(bombi)/40).(ybomb.(i).(bombi)/30) != 4 then 
     explosion i bombi;
   if !bomb_index then
     draw_image bombtab.(5) xbomb.(i).(bombi) ybomb.(i).(bombi)
   else  draw_image bombtab.(4) xbomb.(i).(bombi) ybomb.(i).(bombi);
done

in


let depose_bomb2 tb i bombi xc yc=

   nbomb.(i).(bombi) <- false;
   draw_image safe.(i) xperso.(i) yperso.(i);
   xbomb.(i).(bombi) <- xc;
   ybomb.(i).(bombi) <- yc;
   level.(xbomb.(i).(bombi)/40).(ybomb.(i).(bombi)/30) <- 4;
   if tb = 0 then
   draw_image bombtab.(0)  xbomb.(i).(bombi) ybomb.(i).(bombi)
   else
   draw_image bombtab.(4)  xbomb.(i).(bombi) ybomb.(i).(bombi);
   blit_image safe.(i) xperso.(i) yperso.(i);
   draw_image tabperso.(12).(i) xperso.(i) yperso.(i);
   if tb = 0 then
   let t=Thread.create meanwhile (i,bombi) in ();
   else 
   let t=Thread.create meanwhile2 (i,bombi) in ();

in


let entretienb i =
while true do
Thread.delay 0.0;
match i with
  0 -> let (tb,i,bombio,xc,yc) = isbomb0() in depose_bomb2 tb i bombio xc yc
| 1 -> let (tb,i,bombio,xc,yc) = isbomb1() in depose_bomb2 tb i bombio xc yc
| 2 -> let (tb,i,bombio,xc,yc) = isbomb2() in depose_bomb2 tb i bombio xc yc
| 3 -> let (tb,i,bombio,xc,yc) = isbomb3() in depose_bomb2 tb i bombio xc yc
| _ -> ()
done
in


let prerefresh () =
for jo = 0 to 3 do
if vivant.(jo)=1 then draw_image safe.(jo) xperso.(jo) yperso.(jo)
done
in


let refreshsafe () =
for jo = 0 to 3 do
if vivant.(jo)=1 then 
begin
blit_image safe.(jo) xperso.(jo) yperso.(jo);
draw_image tabperso.(position.(jo)+direction.(jo)).(jo) xperso.(jo) yperso.(jo);
end
done
in

let levelupdt xl yl case =
level.(xl).(yl) <- case;
match case with
   1 -> prerefresh ();
        draw_image isol (xl*40) (yl*30);
        refreshsafe ();
|  9 ->  prerefresh ();
         draw_image itemstab.(0) (xl*40) (yl*30);
         refreshsafe ();
| 10 ->  prerefresh ();
         draw_image itemstab.(1) (xl*40) (yl*30);
         refreshsafe ();
| 11 ->  prerefresh ();
         draw_image itemstab.(2) (xl*40) (yl*30);
         refreshsafe ();
| 12 ->  prerefresh ();
         draw_image itemstab.(3) (xl*40) (yl*30);
         refreshsafe ();
| _ -> ();
in

let entretienlv i=
while true do
Thread.delay 0.0;
match i with
   0 -> let (xl,yl,case)= islevel0() in levelupdt xl yl case
|  1 -> let (xl,yl,case)= islevel1() in levelupdt xl yl case
|  2 -> let (xl,yl,case)= islevel2() in levelupdt xl yl case
|  3 -> let (xl,yl,case)= islevel3() in levelupdt xl yl case
| _ -> ()
done
in

let itemth i=
while true do
Thread.delay 0.0;
match i with
   0 -> let (nj,intens)= isitem0() in intensite.(nj) <- intens
|  1 -> let (nj,intens)= isitem1() in intensite.(nj) <- intens
|  2 -> let (nj,intens)= isitem2() in intensite.(nj) <- intens
|  3 -> let (nj,intens)= isitem3() in intensite.(nj) <- intens
| _ -> ()
done
in


(* on definit la boucle principale *)

let wait_key2 () = 
let pv = getdead () in for jo = 0 to 3 do
vivant.(jo) <- pv.(jo);
done;
for jo = 0 to 3 do
if vivant.(jo) = 1 then
begin
blit_image safe.(jo) xperso.(jo) yperso.(jo);
draw_image tabperso.(12).(jo) xperso.(jo) yperso.(jo);
end;
done;

let t=Thread.create raster (nbjoueur) in 
let t=Thread.create entretienb (nbjoueur) in 
let t=Thread.create entretienlv (nbjoueur) in 
let t=Thread.create itemth (nbjoueur) in 

while true do
 Thread.delay 0.0001;
 if key_pressed() then
 let c = read_key () in
 match (c) with
  'q' ->  vivant.(nbjoueur) <- 2;
          spawn {imoff (nbjoueur)};
          Thread.delay 0.5;
          close_graph();
          Pervasives.exit 0
| '6' -> if vivant.(nbjoueur) = 1 then droite (nbjoueur);
| '4' -> if vivant.(nbjoueur) = 1 then gauche (nbjoueur);
| '8' -> if vivant.(nbjoueur) = 1 then monte (nbjoueur);
| '2' -> if vivant.(nbjoueur) = 1 then descent (nbjoueur);
| ' ' -> if vivant.(nbjoueur) = 1 then
          let w= ref 0 in
          while (!w < !nobomb) do
           if nbomb.(nbjoueur).(!w) then 
                begin
                nbomb.(nbjoueur).(!w) <- false;
                let b = !w in
                let xpo =(xperso.(nbjoueur)/40 *40) in
                let ypo = (yperso.(nbjoueur)/30 *30) in
                spawn {bombis (!tbomb,nbjoueur,b,xpo,ypo)};
                w:= !nobomb+1;
                end
          else w :=!w+1
          done ;
| '0' ->  if (vivant.(nbjoueur) = 1) && (!tbomb = 1) then 
	   let w = ref 0 in
	   while (!w < 9) && (nbomb.(nbjoueur).(!w)) do w:= !w+1 done;
	   if not nbomb.(nbjoueur).(!w) then
       spawn { levelisc (xbomb.(nbjoueur).(!w)/40,ybomb.(nbjoueur).(!w)/30,1)};
| '.' ->  draw_image safe.(nbjoueur) xperso.(nbjoueur) yperso.(nbjoueur);
          xperso.(nbjoueur) <- 40;
          yperso.(nbjoueur) <- 30;
          vivant.(nbjoueur) <- 1;
          direction.(nbjoueur) <- 12;
          refresh_scr (nbjoueur);
(*         spawn {live (nbjoueur)}; *)
|  _  ->()
done;
in wait_key2 ()
  end


let  def  new_game (my_loc,(name:string),(args:string list)) =
 let loc game
  def
 affiche ()=
   reply

 or

 local_init () =
       Join.go my_loc ; 
       affiche ();
       print_int (3);
       print_newline();
       reply
 do {
  (* ce qui est executer par le serveur *)
   print_string "salut je suis le serveur";
   print_newline();

       Join.go my_loc ; start_client ();

  (* ce qui est executer par le client *)
} 
in
(* local_init (); 
   print_int (f 2);
   print_newline();*)
reply
;;

let _ = 
  Arg.parse (Genserver.ns_args ()) (fun _ -> ()) "start jblast server";
  Genserver.server new_game vartype




