(***********************************************************************)
(*                                                                     *)
(*                           xlib for Ocaml                            *)
(*                                                                     *)
(*       Fabrice Le Fessant, projet Para/SOR, INRIA Rocquencourt       *)
(*                                                                     *)
(*  Copyright 1998 Institut National de Recherche en Informatique et   *)
(*  Automatique.  Distributed only by permission.                      *)
(*                                                                     *)
(***********************************************************************)

open Xlib;;
open Xtypes
open Xconst
open Xutils 
open Zpixmap
  
  
let dpy = Xlib.openDisplay ""
let root = defaultRoot dpy
let black= defaultBlack dpy
let white= defaultWhite dpy
let depth = defaultDepth dpy
let cmap = defaultColormap dpy
let gc = X.createGC dpy root []  
  
let rec getint f = 
  let c = input_char f in
  match c with
    'P' | '#' -> 
      let _ = input_line f in
      getint f
  | ' ' | '\n'-> getint f
  | _ -> let a = String.make 3 ' ' in 
      a.[0] <- c;
      let c2 = input_char f in
      match c2 with
        ' ' | '\n' -> a.[2] <- a.[0];
          a.[0] <- '0';
          a.[1] <- '0';
          int_of_string a
      | _ -> a.[1] <- c2;
          let c3 = input_char f in
          match c3 with
            ' ' | '\n' ->
              a.[2] <- a.[1];
              a.[1] <- a.[0];
              a.[0] <- '0';
              int_of_string a
          | _ -> a.[2] <- c3; 
              int_of_string a


let load_ppm filename =
  let f =  open_in filename in
  try
    let dx = getint f in
    let dy = getint f in
    let _ = getint f in
    let size = dx * dy * 3 in
    let image = String.create size in
(*  let _ = input_byte f in *)
    let _ = really_input f image 0 size in
    close_in f;
    dx,dy,image
  with
    e -> close_in f; raise e
    
let convert dpy filename transp =
  let dx,dy,buffer = load_ppm filename in
  let image = createZPixmap dpy dx dy depth in
  for y = 0 to dy - 1 do
    let offset = dx * y * 3 in
    for x = 0 to dx-1 do
      let color =
        if
          Char.code buffer.[offset+3*x] = 255 &&
          Char.code buffer.[offset+3*x+1] = 255 &&
          Char.code buffer.[offset+3*x+2] = 255 then
          transp
        else
        try
          (X.allocColor dpy cmap 
            (257 * (Char.code buffer.[offset+3*x  ] land 0xf0))
            (257 * (Char.code buffer.[offset+3*x+1] land 0xf0))
            (257 * (Char.code buffer.[offset+3*x+2] land 0xf0))).ac_pixel
        with
          _ -> black
      in
      setPixel image x y color
      done
  done;
  image
  
let _ =
  let {anc_pixel = transp} = X.allocNamedColor dpy cmap "cadetblue" in
  let image = convert dpy "case.ppm" transp in 
  putImage dpy root gc 0 0 image ;
(*  for i = 0 to 6 do
  done;
  for i = 0 to 3 do
    let image = convert dpy (Printf.sprintf "bateau%d.ppm" i) transp in ()
(*    putImage dpy root gc 0 0 image *)
  done;
  *)
  let _ = sync dpy in
  Xlib.closeDisplay dpy
  