(***********************************************************************)
(*                                                                     *)
(*                           xlib for Ocaml                            *)
(*                                                                     *)
(*       Fabrice Le Fessant, projet Para/SOR, INRIA Rocquencourt       *)
(*                                                                     *)
(*  Copyright 1998 Institut National de Recherche en Informatique et   *)
(*  Automatique.  Distributed only by permission.                      *)
(*                                                                     *)
(***********************************************************************)

exception ReadOnlyBuffer

type position = int
type delta = int

type direct
type point
type action
type line
type t
type session
type attribute = int
  
type repr =
  { repr_line_pos: int;
    repr_line_len: int;
    mutable repr_attr: attribute;
    repr_charsize: int;
    repr_size: int;
    repr_pos: int }


type line =
  { mutable position: direct;
    mutable representation: repr list;
    mutable modified: int;
    mutable repr_len: int;
    mutable repr_string: string;
    mutable line_hlt : int;}


val dummy_line : line

val create : string -> t

val undo : t -> action * position * delta
val insert_at_end : t -> string -> unit
val insert : t -> point -> string -> position * int
val delete : t -> point -> delta -> position * string

val direct_attr : attribute
val inverse_attr : attribute
val make_attr: int -> int -> int -> bool -> attribute
val set_attr : t -> point -> delta -> attribute -> unit
val unset_attr : t -> unit
    
val find_xy : t -> int -> int -> int -> int * int

val add_point : t -> point
val dup_point : t -> point -> point
val remove_point : t -> point -> unit
val distance : t -> point -> point -> delta
val get_position : t -> point -> position
val set_position : t -> point -> position -> unit
val goto_point : t -> point -> point -> unit

val read : in_channel -> t
val save : t -> out_channel -> unit
val get_char : t -> point -> char
val get_attr : t -> point -> attribute
val set_char_attr : t -> point -> attribute -> unit
val fmove : t -> point -> delta -> delta
val bmove : t -> point -> delta -> delta
val to_string : t -> string
val sub : t -> point -> delta -> string
val clear : t -> unit

val search_forward : t -> Str2.regexp -> point -> delta
val search_forward_matched : t -> Str2.regexp -> point -> string
val search_backward : t -> Str2.regexp -> point -> delta
val search_forward_groups :
  t -> Str2.regexp -> point -> int -> string array
val replace_matched : t -> string -> string

val compute_representation : t -> string array -> int -> line

val point_to_eol : t -> point -> int
val point_to_bol : t -> point -> int
val point_to_eof : t -> point -> int
val point_to_bof : t -> point -> int
val move : t -> point -> int -> int
val point_to_lof : t -> point -> int -> int
val point_to_lol : t -> point -> int -> int
val point_to_line : t -> point -> int -> unit


val point_line : t -> point -> int
val point_col : t -> point -> int
val goto_line : t -> point -> int -> unit

val version : t -> int
val nbre_lines : t -> int
val size : t -> int
val region : t -> point -> point -> string
val update : t -> string -> unit

val lexing : t -> point -> point -> Lexing.lexbuf
val start_session : t -> session
val commit_session : t -> session -> unit
