(***********************************************************************)
(*                                                                     *)
(*                     the join-calculus language                      *)
(*                                                                     *)
(*   Cedric Fournet and Luc Maranget, projet PARA, INRIA Rocquencourt  *)
(*                                                                     *)
(*  Copyright 1997 Institut National de Recherche en Informatique et   *)
(*  Automatique.  Distributed only by permission.                      *)
(*                                                                     *)
(***********************************************************************)

open Graphics
open Join

let _ = open_graph (" 800x800")

let sx = size_x ()
let sy = size_y ()


let xmin = float_of_string("-1.5")
and xmax =  float_of_string("0.5")
and ymin =  float_of_string("-1.25")
and ymax =  float_of_string("+1.25")

let pixel = 1

let limit = 32

let _ = set_font("-*-times-bold-r-*-*-20-*-*-*-*-*-*-*")

let def print_name (i,j,w,h,name,color) =
    Debug.print ("name of window "^name);
    (let wt,ht = text_size(name) in
      if color <> black then (
          set_color(black) ;      
          fill_rect (i+2) (j+h/2-2) (wt+4) (ht+4)
        );
      set_color(color) ;
      moveto (i+4) (j+h/2-4) ;
      draw_string(name) );
    print_string ("Client : "^name);
    print_newline();
    reply ()
;;

let def draw_square (name,i,j,w,h,data)  =
    {
      Debug.print ("----------> "^name);
      Draw.quad data i j w h limit ;
      print_name (i,j,w,h,name,white);} |
      reply  to draw_square

let def make_comp (there) =
    let loc mandel [Quad;Calc] def
          square (i0,j0,w,h) =
          let r = Quad.empty w h limit in
          let def iter (i,j) =
              let ii = i+i0 and jj = j+j0 in
              if i >= w then
                reply ()
              else 
              if j >= h then
                reply iter ( (i+1),0 )
              else
                {
                  let x = xmin +. ((xmax -. xmin) *. float(ii*pixel)) /. float(sx)
                  and y = ymin +. ((ymax -. ymin) *. float(jj*pixel)) /. float(sy) in
                  let m = Calc.mandel x y limit in
                  Quad.set r i j m;
                  reply iter (i,j+1)
                } 
          in
          iter (0,0) ;reply r to square
        and kill_me! () = print_string("Here I die") ; print_newline () ; 
          kill top_location;
      do {
          print_string("Here I go") ; print_newline () ;
          go there  ; print_string("Here I am") ; print_newline () ;
        }
    in
    reply (square,kill_me)

let ww = 8
let hh = 8
let w = size_x () / ww and h = size_y () / hh

let def s! (n,m) | next! (name,job,kill) =
    let w = min w (sx-n) and h = min h (sy-m) in
    print_name (n,m,w,h,name,black) ;
    let def finished r | mutex! () = 
        print_string "finished+mutex";
        print_newline ();
        draw_square (name,n,m,w,h,r);
        job_done ();
        next(name,job,kill) | reply
      or restart () | mutex! () = 
          print_string "restart+mutex";
          print_newline ();
          s(n,m) | reply
    in
    mutex () |
      loc boss do {
          { fail job;
            print_string "failed";
            print_newline ();
            restart (); halt (); } |
          let r = job (n/pixel,m/pixel,w/pixel,h/pixel) in
          print_string "job done";
          print_newline ();
          finished r;
          halt ();
        }
  or killAll! () | next! (name,job,kill) = killAll() | kill() 
  and counter! n | job_done () =
    { if ww*hh = n+1 then killAll () else counter (n+1) } | reply ()

let def register_me (local,name,(args:string list)) =
    reply () |
      let name =
        match args with
	| s :: l -> s
	| [] -> name
      in
      let name =
	if String.length(name) > 8 then
          String.sub name 0 8
        else name in
      let job,kill = make_comp (local) in
      Debug.print ("next_name "^name) ; next(name,job,kill)


let _ = 
  spawn { counter 0 };
  for i = 0 to ww - 1 do
    for j = 0 to hh - 1 do
      spawn { s(i*w,j*w) }
    done
  done

let _ = 
  Arg.parse (Genserver.ns_args ()) (fun _ -> ()) "Start tron server";
  Genserver.server register_me vartype
  
