open Jtypes

(* Attention :                                                                      *)
(* gerer Some et None dans les listes de stubs avant d'essayer de virer les Some    *)


let _ = CdDebug.cadmiumFlag := true
let char = 'z'


(* Operation on objects uids *)

let get_uid (stub:Obj.t) = (* extracts the object uid from its stub *)
  let (locator,_,_,_,_) = Obj.magic stub
  in let (Some (_,_,(uid:Jtypes.uid),_)) = Obj.magic locator
  in uid

let print_uid uid = (* prints a given uid *)
  let (Jtypes.Uid (space_id,obj_num)) = uid
  in Debug.print2 char "uid %s %d\n" 
       (Space.string_of_space_id space_id) obj_num

let print_uids stublist = (* prints all object uids of a stub list *)
  match stublist with
      [] -> Debug.print0 char "print_uids: empty stublist\n"
    | l -> (
  	CdDebug.print "print_uids: ";
  	let give_list stub =
	  print_uid (get_uid stub)
  	in List.iter give_list stublist)

(* Operation on space tables *)

let print_spaces () =
 (* (addr: Unix.inet_addr) (port: Jtypes.port) (space:Jtypes.space) *)
  let give_list (addr,(port: Jtypes.port)) space = 
    CdDebug.print "";
    Debug.print3 char "Addr: %s, Port: %s, Space:%s\n" 
      (Unix.string_of_inet_addr addr) (string_of_int port) 
      (Space.string_of_space_id space.Jtypes.space_id)
  in Hashtbl.iter give_list Space.spaces

let list_spaces () =                  (* known spaces *)
  let known_spaces = ref []
  in (let make_list (addr,(port: Jtypes.port)) space = 
       	known_spaces := space::(!known_spaces);
(* 	CdDebug.print "Added: "; *)
	Debug.print3 char "Addr: %s, Port: %s, Space:%s\n"
	  (Unix.string_of_inet_addr addr) (string_of_int port) 
	  (Space.string_of_space_id space.Jtypes.space_id);
      in Hashtbl.iter make_list Space.spaces);
    (known_spaces)



(* Operation on name server tables *)

let list_servers () =                  (* known servers *)
  let known_servers = ref []
  in (let make_list name (obj,objtype) =
	known_servers := (name,(obj,objtype))::(!known_servers);
(* 	CdDebug.print "Added: "; *)
	Debug.print1 char "Server: %s\n" name;
      in Hashtbl.iter make_list Space.name_server);
    (known_servers)



(* Operation on stub tables *)

let list_gc_stubs space =           (* Gc_stubs *)
  let gc_stub_list = ref []
  in 
    (let make_list (uid,int) (Some Some gc_stub) =
       gc_stub_list := gc_stub::(!gc_stub_list)
     in Whashtbl.iter make_list space.Jtypes.stubs_table);
    (gc_stub_list)

let list_object_gc_stubs space =    (* Gc_stubs for objects only (Tag.object = 244) *)
  let gc_stub_list = ref []
  in 
    (let make_list (uid,int) (Some Some gc_stub) =
       (let (ref_stub,_,_) = gc_stub
	in
	  if (Obj.tag ref_stub) = Tag.object
	  then gc_stub_list := gc_stub::(!gc_stub_list))
     in Whashtbl.iter make_list space.Jtypes.stubs_table);
    (gc_stub_list)

let list_object_stubs space =       (* Stubs for objects accessible from the outside only *)
(*   CdDebug.print "list_object_stubs: treat space: "; *)
  Debug.print0 char "list_object_stubs: treat space: ";
  (let ((addr,port),_) = space.Jtypes.space_id
   in 
     Debug.print3 char "Addr: %s, Port: %s, Space:%s\n" 
       (Unix.string_of_inet_addr addr) (string_of_int port) 
       (Space.string_of_space_id space.Jtypes.space_id));
  let stub_list = ref []
  in 
    (let make_list (uid,int) (Some Some gc_stub) =
       (let (ref_stub,_,_) = gc_stub
	in
	  if (Obj.tag ref_stub) = Tag.object
	  then stub_list := ref_stub::(!stub_list))
     in Whashtbl.iter make_list space.Jtypes.stubs_table);
    (stub_list)

let list_all_object_stubs spaces =       (* Stubs for objects accessible from the outside only *)
                                         (* and from all known spaces *)
(*   CdDebug.print "list_all_object_stubs\n"; *)
  Debug.print0 char "list_all_object_stubs\n";
  let stub_lists = ref [[]]
  in 
    (let make_list space =
       stub_lists := (!(list_object_stubs space))::(!stub_lists)
     in List.iter make_list spaces);
    let stub_lists := List.flatten (!stub_lists)
    in
      Debug.print0 char "list_all_object_stubs: "; print_uids (!stub_lists);Debug.print0 char "\n";
      (stub_lists)
    
let list_stubs () =                   (* Stubs *)
  let stub_list = ref []
  in (let make_list uid (Some stub) =
     	stub_list := stub::(!stub_list)
      in Whashtbl.iter make_list Space.stubs_table);
    (stub_list)



(* Operation on scion tables *)

let list_gc_scions space =           (* Gc_scions *)
  let gc_scion_list = ref []
  in 
    (let make_list (uid,int) gc_scion =
       gc_scion_list := gc_scion::(!gc_scion_list)
     in Hashtbl.iter make_list space.Jtypes.scions_table);
    (gc_scion_list)

let list_scions () =                   (* Scions *)
  let scion_list = ref []
  in (let make_list uid scion =
     	scion_list := scion::(!scion_list)
      in Hashtbl.iter make_list Space.scions_table);
    (scion_list)






