(***********************************************************************)
(*                                                                     *)
(*                    Join : jutils.mli                                *)
(*                                                                     *)
(*      Fabrice Le Fessant, projet SOR/PARA INRIA Rocquencourt         *)
(*                                                                     *)
(*                 mail: fabrice.le_fessant@inria.fr                   *)
(*                 web:  http://www-sor.inria.fr/~lefessan             *)
(*                                                                     *)
(*  Copyright 1997 Institut National de Recherche en Informatique et   *)
(*  Automatique.  Distributed only by permission.                      *)
(*                                                                     *)
(***********************************************************************)

(*
  We re-implement in_channel and out_channel to allow them to be sent in
  Join messages. They act as remote-channels, ie they are local to there 
  runtime.

*)

val go : Join.location -> unit
val kill : Join.location -> unit
val halt : unit -> 'a
val top_location : Join.location
val getRemoteService : Join.space -> string -> 'b metatype -> 'b
val setLocalService : string -> 'c -> 'c metatype -> unit
val getLocalService : string -> 'd metatype -> 'd
val space_of_addr : Unix.inet_addr -> int -> Join.space
val space_of_string : string -> int -> Join.space
val dgc_stop : unit -> unit
val dgc_restart : unit -> unit
val goo : <  > -> unit
val fail : 'e -> unit
val new_port : int -> unit
val migrate : <  > -> Join.location -> unit
val migrate_to : <  > -> Join.location -> unit

(*** Input/output *)


val jopen_in : string -> in_channel
        (* Open the named file for reading, and return a new input channel
           on that file, positionned at the beginning of the file.
           Raise [Sys_error] if the file could not be opened. *)
val jopen_in_bin : string -> in_channel
        (* Same as [open_in], but the file is opened in binary mode,
           so that no translation takes place during reads. On operating
           systems that do not distinguish between text mode and binary
           mode, this function behaves like [open_in]. *)
val jopen_in_gen : open_flag list -> int -> string -> in_channel
        (* [open_in_gen mode rights filename] opens the file named
           [filename] for reading, as above. The extra arguments
           [mode] and [rights] specify the opening mode and file permissions.
           [open_in] and [open_in_bin] are special cases of this function. *)

val jopen_out : string -> out_channel
        (* Open the named file for writing, and return a new output channel
           on that file, positionned at the beginning of the file. The
           file is truncated to zero length if it already exists. It
           is created if it does not already exists.
           Raise [Sys_error] if the file could not be opened. *)
val jopen_out_bin : string -> out_channel
        (* Same as [open_out], but the file is opened in binary mode,
           so that no translation takes place during writes. On operating
           systems that do not distinguish between text mode and binary
           mode, this function behaves like [open_out]. *)
val jopen_out_gen : open_flag list -> int -> string -> out_channel
        (* [open_out_gen mode rights filename] opens the file named
           [filename] for writing, as above. The extra argument [mode]
           specify the opening mode. The extra argument [rights] specifies
           the file permissions, in case the file must be created.
           [open_out] and [open_out_bin] are special cases of this function. *)

val remote_from_in_channel : in_channel -> in_channel
val remote_from_out_channel : out_channel -> out_channel
