(***********************************************************************)
(*                                                                     *)
(*                            JoCaml                                   *)
(*                                                                     *)
(*       Fabrice Le Fessant, projet Para/SOR, INRIA Rocquencourt       *)
(*                                                                     *)
(*  Copyright 1999 Institut National de Recherche en Informatique et   *)
(*  Automatique.  Distributed only by permission.                      *)
(*                                                                     *)
(***********************************************************************)

open Unix
open ThreadUnix
open Services
open Ns_types

let use_multicast = use_multicast
let name = name
let port = port
let user = ref (try Sys.getenv "USER" with _ -> "pub")
  
let ns_request request =
  (* Find the name server *)
  let server_name, server_port =
    try
      if !use_multicast then begin
          Printf.printf "Using multicast"; print_newline ();
          Multicast.send_and_recv !mgroup !mport mquery
        end else raise Not_found
    with e -> 
        Printf.printf "Multicast aborted (%s)" (Printexc.to_string e);
        print_newline ();
        !name, !port
  in  
  Printf.printf "Querying name server %s:%d" server_name server_port;
  print_newline () ;
  let server_addr = first_addr (gethostbyname server_name) in

  (* Connect to the name server *)
  let s = socket PF_INET SOCK_STREAM 0 in
  connect s  (ADDR_INET (server_addr, server_port));
  let inc = in_channel_of_descr s in
  let outc = out_channel_of_descr s in
  output_string outc connection_header;
  output_value outc request;
  flush outc;
  let rep = input_value inc in
  shutdown s SHUTDOWN_ALL;
  rep

let exc_to_string e =
  match e with
    Unix_error (error, f, arg) -> 
      Printf.sprintf "%s: %s %s" f (Unix.error_message error) arg
  | _ -> Printexc.to_string e
  
let lookup name vtype =
  try
    let name = Filename.concat !user name in
    match ns_request (Lookup name) with
      None -> raise Not_found
    | Some space_id ->
        let space = Space.get_space space_id in
        Services.getRemoteService space name vtype
  with e -> 
      let msg = Printf.sprintf "Lookup [%s] has failed with %s" name
          (exc_to_string e) in
      failwith msg

let register name vtype =
  try
    let name = Filename.concat !user name in
    ns_request (Register (name, Space.local_id));
    setLocalService name vtype
  with e -> failwith (Printf.sprintf "Register [%s] has failed with %s" name
          (exc_to_string e))

let fork_client x =
  let t = (x^" -c -host "^(!name)^" -port "^(string_of_int(!port))) in
  let z = Unix.open_process_out t in
  print_string("forked a new runtime: "^x);
  print_newline() 
