(***********************************************************************)
(*                                                                     *)
(*                           Objective Caml                            *)
(*                                                                     *)
(*            Xavier Leroy, projet Cristal, INRIA Rocquencourt         *)
(*                                                                     *)
(*  Copyright 1996 Institut National de Recherche en Informatique et   *)
(*  Automatique.  Distributed only by permission.                      *)
(*                                                                     *)
(***********************************************************************)

(* $Id: printexc.mli,v 1.2 2000/04/18 14:53:05 lefessan Exp $ *)

(* Module [Printexc]: a catch-all exception handler *)

val add_printer: (exn -> string) -> unit
(* [Printexc.add_printer fn] add a new exception printer. The printer
takes the exception as argument and returns the corresponding string. 
It raises a Not_found exception if it cannot print the exception. *)
  
val catch: ('a -> 'b) -> 'a -> 'b
        (* [Printexc.catch fn x] applies [fn] to [x] and returns the result.
           If the evaluation of [fn x] raises any exception, the
           name of the exception is printed on standard error output,
           and the programs aborts with exit code 2.
           Typical use is [Printexc.catch main ()], where [main], with type
           [unit->unit], is the entry point of a standalone program.
           This catches and reports any exception that escapes the program. *)

val print: ('a -> 'b) -> 'a -> 'b
        (* Same as [catch], but re-raise the stray exception after
           printing it, instead of aborting the program. *)

val to_string : exn -> string
        (* [Printexc.to_string e] returns a string representation of [e]. *)
